//-----------------------------------------------------------------------------
// Copyright:      RAD Electronic Co. LTD,
// Author:         Sh. Nourbakhsh Rad
// Remarks:        
// known Problems: none
// Version:        2.5.0
// Description:    N96 TFT-LCD display driver == 8bit & 16bit mode!
//-----------------------------------------------------------------------------

#include "ili9325.h"

//******************* Constants
unsigned short							TFT_ID_code				= 0x0000;

int 												TFT_margin_xl 		= 0;
int 												TFT_margin_xr 		= TFT_SIZE_X - 1;
int 												TFT_margin_yu 		= 0;
int 												TFT_margin_yl 		= TFT_SIZE_Y - 1;

unsigned int 								newX							= 0;
unsigned int 								newY							= 0;

//*************************************************
//******************* Functions *******************
//*************************************************
void delay200ns(unsigned char n)													//simple delay, about 200ns
{
	while(n--)
	{	
		//---  3 NOPs for about 200ns at 16MHz?  ---//
		_NOP();
		_NOP();
		_NOP();
	}
} //delay200ns

void TFTPortInit(void)																		//initial LCD hardware connections
{
	#if TFT_PORT_INTERFACE
		TFT_DB_init();										//Output & non-pullup

		TFT_CS_init();										//Output & high
		TFT_RS_init();										//Output & high
		TFT_RD_init();										//Output & high
		TFT_WR_init();										//Output & high
	#else
    MCUCR = _BV(SRE) | _BV(SRW);			// enable external SRAM (memory mapped TFT) and one wait state
	#endif
	
	TFT_RST_init();											//Output & low
	TFT_BKL_init();											//Output & low
}//TFTPortInit

void TFTReset(void) 																			//pulse reset signal to LCD
{
	TFT_RST(LOW);
		_delay_ms(200);
	TFT_RST(HIGH);
		_delay_ms(500);
} //TFTReset

//----------------------------------------------------------------------------
void TFTOutCmd(unsigned short ins) 												//write command to LCD
{
	#if TFT_PORT_INTERFACE
		TFT_DB_CLR();
		TFT_CS(LOW);
		TFT_RS(LOW);
		
		#if TFT_8bit_INTERFACE
			TFT_DB_WR(ins >> 8);
			TFT_WR(LOW);
			TFT_WR(HIGH);			
			//TFT_DB_CLR();
		#endif
		
		TFT_DB_WR(ins);
		TFT_WR(LOW);
		TFT_WR(HIGH);

		TFT_RS(HIGH);
		TFT_CS(HIGH);
	#else
		*(volatile unsigned char *)(TFT_CTRL_ADDR) = (ins >> 8);
		*(volatile unsigned char *)(TFT_CTRL_ADDR) =  ins;
	#endif
} //TFTOutCmd







void TFTOutDat(unsigned short dat) 												//write data to LCD
{
	#if TFT_PORT_INTERFACE
		TFT_DB_CLR();
		TFT_CS(LOW);

		#if TFT_8bit_INTERFACE
			TFT_DB_WR(dat >> 8);
			TFT_WR(LOW);
			TFT_WR(HIGH);
			//TFT_DB_CLR();
		#endif
		
		TFT_DB_WR(dat);
		TFT_WR(LOW);
		TFT_WR(HIGH);

		TFT_CS(HIGH);
	#else
		*(volatile unsigned char *)(TFT_DATA_ADDR) = (dat >> 8);
		*(volatile unsigned char *)(TFT_DATA_ADDR) =  dat;
	#endif
} //TFTOutDat

void TFTOutDat2(unsigned char dath, unsigned char datl) 	//write data to LCD
{
	#if TFT_PORT_INTERFACE
		unsigned short 		dat = 0x0000;

		TFT_DB_CLR();
		TFT_CS(LOW);

		#if TFT_8bit_INTERFACE			
			TFT_DB_WR(dath);
			TFT_WR(LOW);
			TFT_WR(HIGH);

			//TFT_DB_CLR();

			TFT_DB_WR(datl);
			TFT_WR(LOW);
			TFT_WR(HIGH);
 		#else
 			dat  = dath << 8;
 			dat |= datl;
 				
			TFT_DB_WR(dat);
			TFT_WR(LOW);
			TFT_WR(HIGH);
 		#endif

		TFT_CS(HIGH);
	#else
		*(volatile unsigned char *)(TFT_DATA_ADDR) = dath;
		*(volatile unsigned char *)(TFT_DATA_ADDR) = datl;
	#endif	
} //TFTOutDat2

unsigned short TFTInDat(void) 														//read data from LCD
{
	unsigned short 		dat = 0x0000;
	
	#if TFT_PORT_INTERFACE
		TFT_DB_IN();
		TFT_CS(LOW);

		TFT_RD(LOW);
			delay200ns(1);
			dat = TFT_DB_RD();
		TFT_RD(HIGH);

		#if TFT_8bit_INTERFACE
			TFT_RD(LOW);
				dat <<= 8;

				delay200ns(1);
				dat |= TFT_DB_RD();
			TFT_RD(HIGH);
		#endif
		
		TFT_CS(HIGH);
  	TFT_DB_OUT();
	#else
		dat	= *(volatile unsigned char *)(TFT_DATA_ADDR);
		dat <<= 8;
		dat |= *(volatile unsigned char *)(TFT_DATA_ADDR);
	#endif

	return(dat);
} //TFTInDat

void TFTSetReg(unsigned short index, unsigned short val)	//set LCD instruction register
{
	TFTOutCmd(index);
	TFTOutDat(val);
}	//TFTSetReg

unsigned short TFTGetReg(unsigned short index)						//Get LCD instruction register
{
	TFTOutCmd(index);
	return(TFTInDat());
}	//TFTGetReg

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
void TFTInit(void) 																				//initial LCD
{
	TFTPortInit();
		_delay_ms(100);
	TFTReset();

	TFT_ID_code = TFTGetReg(TFT_INS_START_OSC);
	//TFT_ID_code = 0x9325;
	
	if (TFT_ID_code == 0x9325) 		//-----  ILI9325
	{
		
		TFTSetReg(0x00E3, 										0x3008);							//set the internal timing
		TFTSetReg(0x00E7, 										0x0012);							//set the internal timing
		TFTSetReg(0x00EF, 										0x1231);							//set the internal timing
		TFTSetReg(TFT_INS_START_OSC, 					0x0001);							//start oscillator
			                                		
			_delay_ms(100);                 		
		TFTSetReg(TFT_INS_DRIV_OUT_CTRL, 			0x0100);							//set SS, SM
		TFTSetReg(TFT_INS_DRIV_WAV_CTRL,			0x0700);							//set 1 line inversion
		TFTSetReg(TFT_INS_ENTRY_MOD,					TFT_VAL_ENTRY_MOD);		//set GRAM write direction, BGR=0		
		TFTSetReg(TFT_INS_RESIZE_CTRL,				0x0000);							//resize register, no resizing
		TFTSetReg(TFT_INS_DISP_CTRL2, 				0x0202);							//set the back porch and front porch, periods = 2
		TFTSetReg(TFT_INS_DISP_CTRL3,					0x0000);							//set non-display area refresh cycle ISC[3:0]
		TFTSetReg(TFT_INS_DISP_CTRL4,					0x0000);							//FMARK function
		TFTSetReg(TFT_INS_RGB_DISP_IF_CTRL1,	0x0000);							//RGB interface setting, select system interface
		TFTSetReg(TFT_INS_FRM_MARKER_POS, 		0x0000);							//frame marker position
		TFTSetReg(TFT_INS_RGB_DISP_IF_CTRL2, 	0x0000);							//RGB interface polarity

		// ----------- Power On sequence ----------//
		TFTSetReg(TFT_INS_POW_CTRL1,					0x0000);							//SAP, BT[3:0], AP, DSTB, SLP, STB
		TFTSetReg(TFT_INS_POW_CTRL2,					0x0007);							//DC1[2:0], DC0[2:0], VC[2:0]
		TFTSetReg(TFT_INS_POW_CTRL3,					0x0000);							//VREG1OUT voltage
		TFTSetReg(TFT_INS_POW_CTRL4,					0x0013);							//VDV[4:0] for VCOM amplitude
			
			_delay_ms(500);																						//dis-charge capacitor power voltage	
			
		TFTSetReg(TFT_INS_POW_CTRL1,					0x1290);							//SAP, BT[3:0], AP, DSTB, SLP, STB
		TFTSetReg(TFT_INS_POW_CTRL2,					0x0527);							//DC1[2:0], DC0[2:0], VC[2:0]
			
			_delay_ms(100);
		TFTSetReg(TFT_INS_POW_CTRL3,					0x0018);							//internal reference voltage= Vci;
			
			_delay_ms(100);
		TFTSetReg(TFT_INS_POW_CTRL4,					0x1000);							//set VDV[4:0] for VCOM amplitude
		TFTSetReg(TFT_INS_POW_CTRL7,					0x001E);							//set VCM[5:0] for VCOMH
		TFTSetReg(TFT_INS_FRM_RATE_COL_CTRL,	0x000D);							//set Frame Rate

		// ----------- Adjust GRAM ----------//
			_delay_ms(100);
		TFTSetReg(TFT_INS_GRAM_HOR_AD, 				0x0000);
		TFTSetReg(TFT_INS_GRAM_VER_AD, 				0x0000);

		// ----------- Adjust the Gamma Curve ----------//
		TFTSetReg(TFT_INS_GAMMA_CTRL1, 				0x0007);
		TFTSetReg(TFT_INS_GAMMA_CTRL2, 				0x0605);
		TFTSetReg(TFT_INS_GAMMA_CTRL3, 				0x0106);
		TFTSetReg(TFT_INS_GAMMA_CTRL4, 				0x0206);
		TFTSetReg(TFT_INS_GAMMA_CTRL5, 				0x0808);
		TFTSetReg(TFT_INS_GAMMA_CTRL6, 				0x0007);
		TFTSetReg(TFT_INS_GAMMA_CTRL7, 				0x0201);
		TFTSetReg(TFT_INS_GAMMA_CTRL8, 				0x0007);
		TFTSetReg(TFT_INS_GAMMA_CTRL9, 				0x0602);
		TFTSetReg(TFT_INS_GAMMA_CTRL10, 			0x0808);
		
		//------------------ Set GRAM area ---------------//
		TFTSetReg(TFT_INS_HOR_START_AD, 			0x0000);							// Horizontal GRAM Start Address
		TFTSetReg(TFT_INS_HOR_END_AD, 				0x00EF);							// Horizontal GRAM End Address
		TFTSetReg(TFT_INS_VER_START_AD, 			0x0000);							// Vertical GRAM Start Address
		TFTSetReg(TFT_INS_VER_END_AD, 				0x013F);							// Vertical GRAM Start Address
		TFTSetReg(TFT_INS_GATE_SCAN_CTRL1, 		0xA700);							// Gate Scan Line  A700
		TFTSetReg(TFT_INS_GATE_SCAN_CTRL2, 		0x0001);							// NDL, VLE, REV
		TFTSetReg(TFT_INS_GATE_SCAN_CTRL3, 		0x0000);							// set scrolling line

		//-------------- Partial Display Control ---------//
		TFTSetReg(TFT_INS_PART_IMG1_DISP_POS, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG1_START_AD, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG1_END_AD, 	0x0000);
		TFTSetReg(TFT_INS_PART_IMG2_DISP_POS, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG2_START_AD, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG2_END_AD, 	0x0000);

		//-------------- Panel Control -------------------//
		TFTSetReg(TFT_INS_PANEL_IF_CTRL1, 		0x0010);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL2, 		0x0000);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL3, 		0x0003);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL4, 		0x0110);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL5, 		0x0000);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL6, 		0x0000);
		
		TFTSetReg(TFT_INS_DISP_CTRL1, 				0x0133);							//262K color, normal display, base image display oprate and turns off partial image
	} //-----  ILI9325
	
	else if (TFT_ID_code == 0x9320) 		//-----  ILI9320
	{
		
		TFTSetReg(0x00E5, 										0x8000);
		TFTSetReg(TFT_INS_START_OSC, 					0x0001);
			                                		
			_delay_ms(100);                 		
		TFTSetReg(TFT_INS_DRIV_OUT_CTRL, 			0x0100);
		TFTSetReg(TFT_INS_DRIV_WAV_CTRL,			0x0700);
		TFTSetReg(TFT_INS_ENTRY_MOD,					TFT_VAL_ENTRY_MOD);
		TFTSetReg(TFT_INS_RESIZE_CTRL,				0x0000);
		TFTSetReg(TFT_INS_DISP_CTRL2, 				0x0202);
		TFTSetReg(TFT_INS_DISP_CTRL3,					0x0000);
		TFTSetReg(TFT_INS_DISP_CTRL4,					0x0000);
		TFTSetReg(TFT_INS_RGB_DISP_IF_CTRL1,	0x0000);
		TFTSetReg(TFT_INS_FRM_MARKER_POS, 		0x0000);
		TFTSetReg(TFT_INS_RGB_DISP_IF_CTRL2, 	0x0000);

		// ----------- Power On sequence ----------//
		TFTSetReg(TFT_INS_POW_CTRL1,					0x0000);
		TFTSetReg(TFT_INS_POW_CTRL2,					0x0000);
		TFTSetReg(TFT_INS_POW_CTRL3,					0x0000);
		TFTSetReg(TFT_INS_POW_CTRL4,					0x0000);
			
			_delay_ms(500);
			
		TFTSetReg(TFT_INS_POW_CTRL1,					0x17B0);
		TFTSetReg(TFT_INS_POW_CTRL2,					0x0137);
			
			_delay_ms(100);
		TFTSetReg(TFT_INS_POW_CTRL3,					0x013C);
			
			_delay_ms(100);
		TFTSetReg(TFT_INS_POW_CTRL4,					0x1400);
		TFTSetReg(TFT_INS_POW_CTRL7,					0x0007);

		// ----------- Adjust GRAM ----------//
			_delay_ms(100);
		TFTSetReg(TFT_INS_GRAM_HOR_AD, 				0x0000);
		TFTSetReg(TFT_INS_GRAM_VER_AD, 				0x0000);

		// ----------- Adjust the Gamma Curve ----------//
		TFTSetReg(TFT_INS_GAMMA_CTRL1, 				0x0007);
		TFTSetReg(TFT_INS_GAMMA_CTRL2, 				0x0504);
		TFTSetReg(TFT_INS_GAMMA_CTRL3, 				0x0703);
		TFTSetReg(TFT_INS_GAMMA_CTRL4, 				0x0002);
		TFTSetReg(TFT_INS_GAMMA_CTRL5, 				0x0707);
		TFTSetReg(TFT_INS_GAMMA_CTRL6, 				0x0406);
		TFTSetReg(TFT_INS_GAMMA_CTRL7, 				0x0006);
		TFTSetReg(TFT_INS_GAMMA_CTRL8, 				0x0404);
		TFTSetReg(TFT_INS_GAMMA_CTRL9, 				0x0700);
		TFTSetReg(TFT_INS_GAMMA_CTRL10, 			0x0A08);
		
		//------------------ Set GRAM area ---------------//
		TFTSetReg(TFT_INS_HOR_START_AD, 			0x0000);
		TFTSetReg(TFT_INS_HOR_END_AD, 				0x00EF);
		TFTSetReg(TFT_INS_VER_START_AD, 			0x0000);
		TFTSetReg(TFT_INS_VER_END_AD, 				0x013F);
		TFTSetReg(TFT_INS_GATE_SCAN_CTRL1, 		0x2700);
		TFTSetReg(TFT_INS_GATE_SCAN_CTRL2, 		0x0001);
		TFTSetReg(TFT_INS_GATE_SCAN_CTRL3, 		0x0000);

		//-------------- Partial Display Control ---------//
		TFTSetReg(TFT_INS_PART_IMG1_DISP_POS, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG1_START_AD, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG1_END_AD, 	0x0000);
		TFTSetReg(TFT_INS_PART_IMG2_DISP_POS, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG2_START_AD, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG2_END_AD, 	0x0000);

		//-------------- Panel Control -------------------//
		TFTSetReg(TFT_INS_PANEL_IF_CTRL1, 		0x0010);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL2, 		0x0000);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL3, 		0x0003);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL4, 		0x0110);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL5, 		0x0000);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL6, 		0x0000);
		
		TFTSetReg(TFT_INS_DISP_CTRL1, 				0x0173);
	} //-----  ILI9320

	else if (TFT_ID_code == 0x9328) 		//-----  ILI9328
	{
		
		TFTSetReg(0x00E5, 										0x8000);
		TFTSetReg(TFT_INS_START_OSC, 					0x0001);
			                                		
			_delay_ms(100);                 		
		TFTSetReg(TFT_INS_DRIV_OUT_CTRL, 			0x0100);
		TFTSetReg(TFT_INS_DRIV_WAV_CTRL,			0x0700);
		TFTSetReg(TFT_INS_ENTRY_MOD,					TFT_VAL_ENTRY_MOD);
		TFTSetReg(TFT_INS_RESIZE_CTRL,				0x0000);
		TFTSetReg(TFT_INS_DISP_CTRL2, 				0x0202);
		TFTSetReg(TFT_INS_DISP_CTRL3,					0x0000);
		TFTSetReg(TFT_INS_DISP_CTRL4,					0x0000);
		TFTSetReg(TFT_INS_RGB_DISP_IF_CTRL1,	0x0000);
		TFTSetReg(TFT_INS_FRM_MARKER_POS, 		0x0000);
		TFTSetReg(TFT_INS_RGB_DISP_IF_CTRL2, 	0x0000);

		// ----------- Power On sequence ----------//
		TFTSetReg(TFT_INS_POW_CTRL1,					0x0000);
		TFTSetReg(TFT_INS_POW_CTRL2,					0x0000);
		TFTSetReg(TFT_INS_POW_CTRL3,					0x0000);
		TFTSetReg(TFT_INS_POW_CTRL4,					0x0000);
			
			_delay_ms(500);
			
		TFTSetReg(TFT_INS_POW_CTRL1,					0x17B0);
		TFTSetReg(TFT_INS_POW_CTRL2,					0x0137);
			
			_delay_ms(100);
		TFTSetReg(TFT_INS_POW_CTRL3,					0x013C);
			
			_delay_ms(100);
		TFTSetReg(TFT_INS_POW_CTRL4,					0x1400);
		TFTSetReg(TFT_INS_POW_CTRL7,					0x0007);

		// ----------- Adjust GRAM ----------//
			_delay_ms(100);
		TFTSetReg(TFT_INS_GRAM_HOR_AD, 				0x0000);
		TFTSetReg(TFT_INS_GRAM_VER_AD, 				0x0000);

		// ----------- Adjust the Gamma Curve ----------//
		TFTSetReg(TFT_INS_GAMMA_CTRL1, 				0x0007);
		TFTSetReg(TFT_INS_GAMMA_CTRL2, 				0x0504);
		TFTSetReg(TFT_INS_GAMMA_CTRL3, 				0x0703);
		TFTSetReg(TFT_INS_GAMMA_CTRL4, 				0x0002);
		TFTSetReg(TFT_INS_GAMMA_CTRL5, 				0x0707);
		TFTSetReg(TFT_INS_GAMMA_CTRL6, 				0x0406);
		TFTSetReg(TFT_INS_GAMMA_CTRL7, 				0x0006);
		TFTSetReg(TFT_INS_GAMMA_CTRL8, 				0x0404);
		TFTSetReg(TFT_INS_GAMMA_CTRL9, 				0x0700);
		TFTSetReg(TFT_INS_GAMMA_CTRL10, 			0x0A08);
		
		//------------------ Set GRAM area ---------------//
		TFTSetReg(TFT_INS_HOR_START_AD, 			0x0000);
		TFTSetReg(TFT_INS_HOR_END_AD, 				0x00EF);
		TFTSetReg(TFT_INS_VER_START_AD, 			0x0000);
		TFTSetReg(TFT_INS_VER_END_AD, 				0x013F);
		TFTSetReg(TFT_INS_GATE_SCAN_CTRL1, 		0x2700);
		TFTSetReg(TFT_INS_GATE_SCAN_CTRL2, 		0x0001);
		TFTSetReg(TFT_INS_GATE_SCAN_CTRL3, 		0x0000);

		//-------------- Partial Display Control ---------//
		TFTSetReg(TFT_INS_PART_IMG1_DISP_POS, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG1_START_AD, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG1_END_AD, 	0x0000);
		TFTSetReg(TFT_INS_PART_IMG2_DISP_POS, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG2_START_AD, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG2_END_AD, 	0x0000);

		//-------------- Panel Control -------------------//
		TFTSetReg(TFT_INS_PANEL_IF_CTRL1, 		0x0010);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL2, 		0x0000);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL3, 		0x0003);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL4, 		0x0110);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL5, 		0x0000);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL6, 		0x0000);
		
		TFTSetReg(TFT_INS_DISP_CTRL1, 				0x0173);
	} //-----  ILI9328

	else														 		//-----  Others!
	{
		
		TFTSetReg(0x00E3, 										0x3008);
		TFTSetReg(0x00E7, 										0x0012);
		TFTSetReg(0x00EF, 										0x1231);
		TFTSetReg(TFT_INS_START_OSC, 					0x0001);
			                                		
			_delay_ms(100);                 		
		TFTSetReg(TFT_INS_DRIV_OUT_CTRL, 			0x0100);
		TFTSetReg(TFT_INS_DRIV_WAV_CTRL,			0x0700);
		TFTSetReg(TFT_INS_ENTRY_MOD,					TFT_VAL_ENTRY_MOD);
		TFTSetReg(TFT_INS_RESIZE_CTRL,				0x0000);
		TFTSetReg(TFT_INS_DISP_CTRL2, 				0x0202);
		TFTSetReg(TFT_INS_DISP_CTRL3,					0x0000);
		TFTSetReg(TFT_INS_DISP_CTRL4,					0x0000);
		TFTSetReg(TFT_INS_RGB_DISP_IF_CTRL1,	0x0000);
		TFTSetReg(TFT_INS_FRM_MARKER_POS, 		0x0000);
		TFTSetReg(TFT_INS_RGB_DISP_IF_CTRL2, 	0x0000);

		// ----------- Power On sequence ----------//
		TFTSetReg(TFT_INS_POW_CTRL1,					0x0000);
		TFTSetReg(TFT_INS_POW_CTRL2,					0x0007);
		TFTSetReg(TFT_INS_POW_CTRL3,					0x0000);
		TFTSetReg(TFT_INS_POW_CTRL4,					0x0000);
			
			_delay_ms(500);
			
		TFTSetReg(TFT_INS_POW_CTRL1,					0x1690);
		TFTSetReg(TFT_INS_POW_CTRL2,					0x0007);
			
			_delay_ms(100);
		TFTSetReg(TFT_INS_POW_CTRL3,					0x001A);
			
			_delay_ms(100);
		TFTSetReg(TFT_INS_POW_CTRL4,					0x1800);
		TFTSetReg(TFT_INS_POW_CTRL7,					0x002A);

		// ----------- Adjust GRAM ----------//
			_delay_ms(100);
		TFTSetReg(TFT_INS_GRAM_HOR_AD, 				0x0000);
		TFTSetReg(TFT_INS_GRAM_VER_AD, 				0x0000);

		// ----------- Adjust the Gamma Curve ----------//
		TFTSetReg(TFT_INS_GAMMA_CTRL1, 				0x0000);
		TFTSetReg(TFT_INS_GAMMA_CTRL2, 				0x0000);
		TFTSetReg(TFT_INS_GAMMA_CTRL3, 				0x0000);
		TFTSetReg(TFT_INS_GAMMA_CTRL4, 				0x0000);
		TFTSetReg(TFT_INS_GAMMA_CTRL5, 				0x0000);
		TFTSetReg(TFT_INS_GAMMA_CTRL6, 				0x0406);
		TFTSetReg(TFT_INS_GAMMA_CTRL7, 				0x0006);
		TFTSetReg(TFT_INS_GAMMA_CTRL8, 				0x0404);
		TFTSetReg(TFT_INS_GAMMA_CTRL9, 				0x0700);
		TFTSetReg(TFT_INS_GAMMA_CTRL10, 			0x0A08);
		
		//------------------ Set GRAM area ---------------//
		TFTSetReg(TFT_INS_HOR_START_AD, 			0x0000);
		TFTSetReg(TFT_INS_HOR_END_AD, 				0x00EF);
		TFTSetReg(TFT_INS_VER_START_AD, 			0x0000);
		TFTSetReg(TFT_INS_VER_END_AD, 				0x013F);
		TFTSetReg(TFT_INS_GATE_SCAN_CTRL1, 		0x2700);
		TFTSetReg(TFT_INS_GATE_SCAN_CTRL2, 		0x0001);
		TFTSetReg(TFT_INS_GATE_SCAN_CTRL3, 		0x0000);

		//-------------- Partial Display Control ---------//
		TFTSetReg(TFT_INS_PART_IMG1_DISP_POS, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG1_START_AD, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG1_END_AD, 	0x0000);
		TFTSetReg(TFT_INS_PART_IMG2_DISP_POS, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG2_START_AD, 0x0000);
		TFTSetReg(TFT_INS_PART_IMG2_END_AD, 	0x0000);

		//-------------- Panel Control -------------------//
		TFTSetReg(TFT_INS_PANEL_IF_CTRL1, 		0x0010);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL2, 		0x0000);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL3, 		0x0003);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL4, 		0x0110);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL5, 		0x0000);
		TFTSetReg(TFT_INS_PANEL_IF_CTRL6, 		0x0000);

		TFTSetReg(TFT_INS_DISP_CTRL1, 				0x0133);

		TFTSetReg(TFT_INS_FRM_RATE_COL_CTRL, 	0x000D);
	} //-----  Others!
} //TFTInit

unsigned short TFTGetID(void)
{
	return(TFT_ID_code);
}	//TFTGetID

void TFTBacklight(unsigned char x)
{
	TFT_BKL(x);
}	//TFTBacklight

void TFTSetArea(int sx,int ex,int sy,int ey)
{
	if (sx < TFT_margin_xl)    				sx = TFT_margin_xl;
	if (ex > TFT_margin_xr)    				ex = TFT_margin_xr;
	if (sy < TFT_margin_yu)    				sy = TFT_margin_yu;
	if (ey > TFT_margin_yl)    				ey = TFT_margin_yl;
	newX = ex - sx + 1;
	
	TFTSetReg(TFT_INS_START_ADX,	sx);
	TFTSetReg(TFT_INS_END_ADX,		ex);
	TFTSetReg(TFT_INS_GRAM_ADX,		sx);

	#if !TFT_ORN_PORTRAIT
		sy = TFT_SIZE_Y - 1 - sy;       										// mirror start y address
		ey = TFT_SIZE_Y - 1 - ey;       										// mirror end 	y address
		newY = sy - ey + 1;

		TFTSetReg(TFT_INS_START_ADY,	ey);
		TFTSetReg(TFT_INS_END_ADY,		sy);
		TFTSetReg(TFT_INS_GRAM_ADY,		sy);
	#else
		newY = ey - sy + 1;

		TFTSetReg(TFT_INS_START_ADY,	sy);
		TFTSetReg(TFT_INS_END_ADY,		ey);
		TFTSetReg(TFT_INS_GRAM_ADY,		sy);
	#endif

	TFTOutCmd(TFT_INS_RW_GRAM);
} //TFTSetArea

void TFTResetArea(void)
{
   TFTSetReg(TFT_INS_START_ADX,	0x0000);
   TFTSetReg(TFT_INS_END_ADX,		GetMaxX());

   TFTSetReg(TFT_INS_START_ADY,	0x0000);
   TFTSetReg(TFT_INS_END_ADY,		GetMaxY());
}	//TFTResetArea

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
void TFTGotoXY(int x,int y)
{
	TFTSetReg(TFT_INS_GRAM_ADX, x); // initial settings for the GDDRAM X address in the address counter (AC).		
	TFTSetReg(TFT_INS_GRAM_ADY, y); // initial settings for the GDDRAM Y address in the address counter (AC).
	
	TFTOutCmd(TFT_INS_RW_GRAM);
}	//TFTGotoXY

void TFTPutPixel(int x,int y,unsigned short color,TFT_mode_t mode)
{
	TFTGotoXY(x, y);

	if ((mode == TFT_MODE_NORMAL) || (mode == TFT_MODE_FULL))
	{
		TFTOutDat(color);
	}
	else if (mode == TFT_MODE_INVERSE)
  {
   color = TFTInDat();     															// ignore invalid data
   color = TFTInDat();
   TFTOutDat(~color);
	}
}	//TFTPutPixel

void TFTFillRect(int sx,int ex,int sy,int ey,unsigned short color,TFT_mode_t mode) 		//draw a rectangular
{
	unsigned int 		i, j;
	
	TFTSetArea(sx, ex, sy, ey);
	
	if ((mode == TFT_MODE_NORMAL) || (mode == TFT_MODE_FULL))
	{
		for (j=0; j<newY; j++)
	  	for (i=0; i<newX; i++)
			{
				TFTOutDat(color);
			}
	}
	else if (mode == TFT_MODE_INVERSE)
  {
  	for (j=0; j<newY; j++)
    	for (i=0; i<newX; i++)
      {
      	color = TFTInDat();                 						// ignore invalid data
      	color = TFTInDat();
      	TFTOutDat(~color);
      }
	}
	
	TFTResetArea();
} //TFTFillRect

//----------------------------------------------------------------------------
//show picture from code memory with specific size
void TFTShowPic(int sx,int ex,int sy,int ey,const unsigned short *pic,TFT_mode_t mode)
{
	unsigned long 	k = 0;
	unsigned short 	color;
	unsigned int 		i, j;
	
	TFTSetArea(sx, ex, sy, ey);

  if (mode == TFT_MODE_FULL)
  {
		for (j=0; j<newY; j++)
			for (i=0; i<newX; i++)
			{
				TFTOutDat(_pgm_read_word(pic[k]));
				k++;
			}
	}
	else if (mode == TFT_MODE_NORMAL)
	{
		for (j=0; j<newY; j++)
			for (i=0; i<newX; i++)
			{
				if (_pgm_read_word(pic[k]) == TFT_COL_WHITE)
				{
					color = TFTInDat();                 					// ignore invalid data
					color = TFTInDat();
					TFTOutDat(color);
				}
				else
				{
					TFTOutDat(_pgm_read_word(pic[k]));
				}
				k++;
			}
  }
  
  TFTResetArea();
} //TFTShowPic
