//-----------------------------------------------------------------------------
// Copyright:      RAD Electronic Co. LTD,
// Author:         Sh. Nourbakhsh Rad
// Remarks:        
// known Problems: none
// Version:        2.1.0
// Description:    ILI9320 & ILI9325 TFT-LCD display driver
//-----------------------------------------------------------------------------

#ifndef _ILI9325_H_
	#define _ILI9325_H_

  //#include "..\app_config.h"
	#include "ili9325config.h"

	// Orientation
	#if TFT_ORN_PORTRAIT
		#define TFT_SIZE_X                        240
		#define TFT_SIZE_Y                        320
		#define TFT_VAL_ENTRY_MOD                 0x0030
		#define TFT_INS_GRAM_ADX                  TFT_INS_GRAM_HOR_AD
		#define TFT_INS_GRAM_ADY                  TFT_INS_GRAM_VER_AD
		#define TFT_INS_START_ADX                 TFT_INS_HOR_START_AD
		#define TFT_INS_END_ADX                   TFT_INS_HOR_END_AD
		#define TFT_INS_START_ADY                 TFT_INS_VER_START_AD
		#define TFT_INS_END_ADY                   TFT_INS_VER_END_AD
	#else
		#define TFT_SIZE_X                        320
		#define TFT_SIZE_Y                        240
		#define TFT_VAL_ENTRY_MOD                 0x0028
		#define TFT_INS_GRAM_ADX                  TFT_INS_GRAM_VER_AD
		#define TFT_INS_GRAM_ADY                  TFT_INS_GRAM_HOR_AD
		#define TFT_INS_START_ADX                 TFT_INS_VER_START_AD
		#define TFT_INS_END_ADX                   TFT_INS_VER_END_AD
		#define TFT_INS_START_ADY                 TFT_INS_HOR_START_AD
		#define TFT_INS_END_ADY                   TFT_INS_HOR_END_AD
	#endif
	
	//*       TFT_ORN_PORTRAIT == 0x00					  TFT_ORN_PORTRAIT == 0x01       *//
	//*                                                                          *//
	//*	        																				  ____________           *//
	//*	                      													 \ ---------- \          *//
	//*	      ________________________                   \\          \\          *//
	//*	     \ ___________________    \                  \\          \\          *//
	//*	     \\                   \[] \                  \\  ------  \\          *//
	//*	     \\   ------------    \[] \                  \\       /  \\          *//
	//*	     \\         /         \[] \                  \\     /    \\          *//
	//*	     \\   ------------>   \[] \ 	               \\   /      \\          *//
	//*	     \\___________________\[] \                  \\  ------> \\          *//
	//*	     \________________________\ 								 \\__________\\          *//
	//*	                                               	 \ [][][][][] \          *//
	//*	                                   							 \____________\          *//
	//*	    																																		 *//
	//*     																																		 *//


	// Commands                 		
	#define TFT_INS_START_OSC										0x0000 //data read at this instruction should be 0x0789 --> use for test connection
	#define TFT_INS_DRIV_OUT_CTRL		        		0x0001
	#define TFT_INS_DRIV_WAV_CTRL		        		0x0002
	#define TFT_INS_ENTRY_MOD										0x0003
	#define TFT_INS_RESIZE_CTRL									0x0004
	#define TFT_INS_DISP_CTRL1									0x0007
	#define TFT_INS_DISP_CTRL2									0x0008
	#define TFT_INS_DISP_CTRL3									0x0009
	#define TFT_INS_DISP_CTRL4									0x000A
	#define TFT_INS_RGB_DISP_IF_CTRL1	        	0x000C
	#define TFT_INS_FRM_MARKER_POS		        	0x000D
	#define TFT_INS_RGB_DISP_IF_CTRL2	        	0x000F
	#define TFT_INS_POW_CTRL1										0x0010
	#define TFT_INS_POW_CTRL2										0x0011
	#define TFT_INS_POW_CTRL3										0x0012
	#define TFT_INS_POW_CTRL4										0x0013
	#define TFT_INS_GRAM_HOR_AD									0x0020
	#define TFT_INS_GRAM_VER_AD									0x0021
	#define TFT_INS_RW_GRAM											0x0022
	#define TFT_INS_POW_CTRL7										0x0029
	#define TFT_INS_FRM_RATE_COL_CTRL	        	0x002B
	#define TFT_INS_GAMMA_CTRL1									0x0030
	#define TFT_INS_GAMMA_CTRL2									0x0031
	#define TFT_INS_GAMMA_CTRL3									0x0032
	#define TFT_INS_GAMMA_CTRL4									0x0035 
	#define TFT_INS_GAMMA_CTRL5									0x0036
	#define TFT_INS_GAMMA_CTRL6									0x0037
	#define TFT_INS_GAMMA_CTRL7									0x0038
	#define TFT_INS_GAMMA_CTRL8									0x0039
	#define TFT_INS_GAMMA_CTRL9									0x003C
	#define TFT_INS_GAMMA_CTRL10								0x003D
	#define TFT_INS_HOR_START_AD								0x0050
	#define TFT_INS_HOR_END_AD									0x0051
	#define TFT_INS_VER_START_AD								0x0052
	#define TFT_INS_VER_END_AD									0x0053
	#define TFT_INS_GATE_SCAN_CTRL1		        	0x0060
	#define TFT_INS_GATE_SCAN_CTRL2		        	0x0061
	#define TFT_INS_GATE_SCAN_CTRL3		        	0x006A
	#define TFT_INS_PART_IMG1_DISP_POS	        0x0080
	#define TFT_INS_PART_IMG1_START_AD	        0x0081
	#define TFT_INS_PART_IMG1_END_AD		        0x0082
	#define TFT_INS_PART_IMG2_DISP_POS	        0x0083
	#define TFT_INS_PART_IMG2_START_AD	        0x0084
	#define TFT_INS_PART_IMG2_END_AD		        0x0085
	#define TFT_INS_PANEL_IF_CTRL1		        	0x0090
	#define TFT_INS_PANEL_IF_CTRL2		        	0x0092
	#define TFT_INS_PANEL_IF_CTRL3		        	0x0093
	#define TFT_INS_PANEL_IF_CTRL4		        	0x0095
	#define TFT_INS_PANEL_IF_CTRL5		        	0x0097
	#define TFT_INS_PANEL_IF_CTRL6		        	0x0098
	
	//Colors
	#define RGB2COL(red, green, blue)						((unsigned int)( (( blue  >> 3 )   << 11 ) | \
																															 (( green >> 2 )   << 5  ) | \
																																( red   >> 3 )))
	
	#define TFT_COL_RED													0x001F
	#define TFT_COL_GREEN												0x07E0
	#define TFT_COL_BLUE												0xF800
	#define TFT_COL_YELLOW											0x07FF
	#define TFT_COL_PURPLE											0xF81F
	#define TFT_COL_BLACK												0x0000
	#define TFT_COL_WHITE												0xFFFF
	
	//RGB Colors																			 (  R ,  G  ,  B  ) from PhotoShop!!!
	#define BLACK                       				RGB2COL(0x00, 0x00, 0x00)
	#define GRAY                        				RGB2COL(0x95, 0x95, 0x95)
	#define WHITE                       				RGB2COL(0xFF, 0xFF, 0xFF)

	#define RED                         				RGB2COL(0xFF, 0x00, 0x00)
	#define LIGHT_RED              			 				RGB2COL(0xF2, 0x6C, 0x4E)
	#define DARK_RED          	  			 				RGB2COL(0x79, 0x00, 0x00)
	
	#define GREEN                       				RGB2COL(0x00, 0xFF, 0x00)
	#define LIGHT_GREEN            	    				RGB2COL(0x3B, 0xB8, 0x78)
	#define DARK_GREEN        			    				RGB2COL(0x00, 0x58, 0x25)
	
	#define BLUE                        				RGB2COL(0x00, 0x00, 0xFF)
	#define LIGHT_BLUE                 					RGB2COL(0x00, 0xBF, 0xF3)
	#define DARK_BLUE           	     					RGB2COL(0x00, 0x5B, 0x7F)

	#define YELLOW                      				RGB2COL(0xFF, 0xFF, 0x00)
	#define LIGHT_YELLOW           	    				RGB2COL(0xFF, 0xF4, 0x68)
	#define DARK_YELLOW        			    				RGB2COL(0xAB, 0xA0, 0x00)
	
	#define MAGENTA                     				RGB2COL(0xFF, 0x00, 0xFF)
	#define CYAN                        				RGB2COL(0x00, 0xFF, 0xFF)
	#define BROWN                  		  				RGB2COL(0xA6, 0x7C, 0x51)
	#define SILVER                      				RGB2COL(0xCC, 0xCC, 0xCC)
	#define GOLD                        				RGB2COL(0xD7, 0xCD, 0x19)
	
	//Variables
	typedef enum {
		TFT_MODE_NORMAL,
	  TFT_MODE_INVERSE,
	  TFT_MODE_FULL
	} TFT_mode_t;
	
	extern int 																	TFT_margin_xl;
	extern int 																	TFT_margin_xr;
	extern int 																	TFT_margin_yu;
	extern int 																	TFT_margin_yl;
	
	#define TFT_CLS(clr)                        TFTFillRect(0, (TFT_SIZE_X-1), 0, (TFT_SIZE_Y-1), clr, TFT_MODE_NORMAL)

	#define GetMaxX() 													((unsigned int)TFT_SIZE_X-1)
	#define GetMaxY() 													((unsigned int)TFT_SIZE_Y-1)


	//******************* Function Prototypes
	 void TFTOutCmd(unsigned short ins);
    void TFTOutDat(unsigned short dat); 											//write data to LCD
	void TFTOutDat2(unsigned char dath, unsigned char datl); 	//write data to LCD
    
      	
	unsigned short TFTInDat(void); 														//read data from LCD
	
	void TFTSetReg(unsigned short index,unsigned short val); 	//set LCD instruction register
	unsigned short TFTGetReg(unsigned short index); 					//Get LCD instruction register
	
	//////////////////////////////
	void TFTInit(void); //initial LCD

	void TFTBacklight(unsigned char x);
	unsigned short TFTGetID(void);

	void TFTSetArea(int sx,int ex,int sy,int ey);
	void TFTResetArea(void);
//    void TFTReset(void); 
	//////////////////////////////
	void TFTGotoXY(int x,int y);
	
	void TFTPutPixel(int x,int y,unsigned short color,TFT_mode_t mode);
	void TFTFillRect(int sx,int ex,int sy,int ey,unsigned short color,TFT_mode_t mode); //draw a rectangular
	
	//////////////////////////////
	//show picture from code memory with specific size
	void TFTShowPic(int sx,int ex,int sy,int ey,const unsigned short *pic,TFT_mode_t mode);

#endif //_ILI9325_H_
