//-----------------------------------------------------------------------------
// Copyright:      RAD Electronic Co. LTD,
// Author:         Sh. Nourbakhsh Rad
// Remarks:        
// known Problems: none
// Version:        1.5.0
// Description:    ILI9320 & ILI9325 TFT-LCD display driver
//-----------------------------------------------------------------------------

#ifndef _ILI9325C_H_
	#define _ILI9325C_H_

  #include "..\app_config.h"

	
	#define TFT_ORN_PORTRAIT										1				// 1 Portrait display : 0 Landscape display *** see below	
	#define TFT_PORT_INTERFACE									1				// 1 TFT-LCD port interfaced : 0 TFT-LCD memory-mapped interfaced
	#define TFT_8bit_INTERFACE									1				// 1 TFT-LCD in 8bit mode : 0 TFT-LCD in 16bit mode

	// Pins.....
	#if TFT_PORT_INTERFACE
		//--- DATA ---
		#define TFT_DB_DDR1												DDRC			//D10 to D17 of LCD
		#define TFT_DB_PRT1												PORTC
		#define TFT_DB_PIN1												PINC
		
		#if !TFT_8bit_INTERFACE
			#define TFT_DB_DDR2											DDRB			//D0 to D7 of LCD
			#define TFT_DB_PRT2											PORTB
			#define TFT_DB_PIN2											PINB
		#endif
		
		//--- CTRL ---
		#define TFT_RS_DDR												DDRD
		#define TFT_RS_PRT												PORTD
		#define TFT_RS_BIT												4
		                      										
		#define TFT_CS_DDR												DDRD
		#define TFT_CS_PRT												PORTD
		#define TFT_CS_BIT												2
		                      										
		#define TFT_RD_DDR												DDRD
		#define TFT_RD_PRT												PORTD
		#define TFT_RD_BIT												5
		                      										
		#define TFT_WR_DDR												DDRD
		#define TFT_WR_PRT												PORTD
		#define TFT_WR_BIT												3

		#define TFT_RST_DDR	       								DDRD
		#define TFT_RST_PRT	       								PORTD
		#define TFT_RST_BIT												6
		
		#define TFT_BL_DDR												DDRD
		#define TFT_BL_PRT												PORTD
		#define TFT_BL_BIT												7

	#elif defined(__AVR_AT90S4414__) || defined(__AVR_AT90S8515__) || defined(__AVR_ATmega64__)  || \
 		   	defined(__AVR_ATmega8515__)|| defined(__AVR_ATmega103__) || defined(__AVR_ATmega128__) || \
   		 	defined(__AVR_ATmega161__) || defined(__AVR_ATmega162__)

			#if TFT_8bit_INTERFACE
				#define TFT_CTRL_ADDR											0x9000
				#define TFT_DATA_ADDR											0x9200
		
				#define TFT_RST_DDR	       								DDRD
				#define TFT_RST_PRT	       								PORTD
				#define TFT_RST_BIT												7
				
				#define TFT_BL_DDR												DDRB
				#define TFT_BL_PRT												PORTB
				#define TFT_BL_BIT												5
			#else
				#error "external data memory interface is not 16bit, you can not use TFT-LCD in memory interface and 16bit mode !!!"
			#endif

	#else
		#error "external data memory interface not available for this device, use TFT-LCD port interface mode"
	#endif
	
	//---------------------------------------------------------------
	#if TFT_PORT_INTERFACE			
		#if TFT_8bit_INTERFACE
			#define TFT_DB_WR(x)										TFT_DB_PRT1 = x
			#define TFT_DB_RD()											TFT_DB_PIN1

			#define TFT_DB_OUT()										TFT_DB_DDR1 = 0xFF
			#define TFT_DB_IN()											TFT_DB_DDR1 = 0x00

			#define TFT_DB_CLR()										TFT_DB_PRT1 = 0x00
		 	#define TFT_DB_PLU()										TFT_DB_PRT1 = 0xFF
		#else			
			#define TFT_DB_WR(x)										{TFT_DB_PRT1 = x >> 8; TFT_DB_PRT2 = x;}
			#define TFT_DB_RD()											((TFT_DB_PIN1 << 8) + TFT_DB_PIN2)

			#define TFT_DB_OUT()										{TFT_DB_DDR1 = 0xFF; TFT_DB_DDR2 = 0xFF;}
			#define TFT_DB_IN()											{TFT_DB_DDR1 = 0x00; TFT_DB_DDR2 = 0x00;}

			#define TFT_DB_CLR()										{TFT_DB_PRT1 = 0x00; TFT_DB_PRT2 = 0x00;}
			#define TFT_DB_PLU()										{TFT_DB_PRT1 = 0xFF; TFT_DB_PRT2 = 0xFF;}
		#endif
		
		#define TFT_DB_init()											{TFT_DB_OUT(); TFT_DB_CLR();}

		//-------------------------
		#define TFT_CS_init()											{sbi(TFT_CS_DDR,TFT_CS_BIT); sbi(TFT_CS_PRT,TFT_CS_BIT);}
		#define TFT_RS_init()											{sbi(TFT_RS_DDR,TFT_RS_BIT); sbi(TFT_RS_PRT,TFT_RS_BIT);}
		#define TFT_RD_init()											{sbi(TFT_RD_DDR,TFT_RD_BIT); sbi(TFT_RD_PRT,TFT_RD_BIT);}
		#define TFT_WR_init()											{sbi(TFT_WR_DDR,TFT_WR_BIT); sbi(TFT_WR_PRT,TFT_WR_BIT);}

		#define TFT_CS(x)													(x ? (sbi(TFT_CS_PRT,TFT_CS_BIT)) : (cbi(TFT_CS_PRT,TFT_CS_BIT)))
		#define TFT_RS(x)													(x ? (sbi(TFT_RS_PRT,TFT_RS_BIT)) : (cbi(TFT_RS_PRT,TFT_RS_BIT)))
		#define TFT_RD(x)													(x ? (sbi(TFT_RD_PRT,TFT_RD_BIT)) : (cbi(TFT_RD_PRT,TFT_RD_BIT)))
		#define TFT_WR(x)													(x ? (sbi(TFT_WR_PRT,TFT_WR_BIT)) : (cbi(TFT_WR_PRT,TFT_WR_BIT)))
	#endif
	
	//-------------------------
	#define TFT_RST_init()											{sbi(TFT_RST_DDR,TFT_RST_BIT); cbi(TFT_RST_PRT,TFT_RST_BIT);}
	#define TFT_BKL_init()											{sbi(TFT_BL_DDR,TFT_BL_BIT)  ; cbi(TFT_BL_PRT,TFT_BL_BIT);}
		
	#define TFT_RST(x)													(x ? (sbi(TFT_RST_PRT,TFT_RST_BIT)) : (cbi(TFT_RST_PRT,TFT_RST_BIT)))
	#define TFT_BKL(x)                          (x ? (sbi(TFT_BL_PRT,TFT_BL_BIT))   : (cbi(TFT_BL_PRT,TFT_BL_BIT)))

#endif //_ILI9325C_H_
